def get_choice_input(heading, choices):
    map_choice = False
    while 1:
        input_text = heading + "\n"
        for idx, choice in enumerate(choices, 1):
            if isinstance(choice, list):
                choice_string = choice[0]
                map_choice = True
            else:
                choice_string = choice
            input_text += "   " + str(idx) + ") " + choice_string + "\n"
        choice = input(input_text)

        try:
            choice_value = int(choice)
        except ValueError:
            print(choice + " wasn't a valid number, please try again")
            continue

        if choice_value < 1 or choice_value > len(choices):
            print(choice + " was not one of the options, please try again")
        else:
            if not map_choice:
                return choice_value
            else:
                return choices[choice_value - 1][1]



from pint import UnitRegistry


class Singleton(type):
    """
    Define an Instance operation that lets clients access its unique
    instance.
    """

    def __init__(cls, name, bases, attrs, **kwargs):
        super().__init__(name, bases, attrs)
        cls._instance = None

    def __call__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__call__(*args, **kwargs)
        return cls._instance


class UnitConversion(metaclass=Singleton):
    def __init__(self):

        # Initialize pint's unit registry object
        self.unit_reg = UnitRegistry()

        self.metre_per_second = self.unit_reg.metre / self.unit_reg.second

    def get_unit_registry(self):
        return self.unit_reg



import unittest

from pepys_import.core.formats.state import State
from pepys_import.core.formats import unit_registry, quantity


class UnitsTests(unittest.TestCase):
    def setUp(self):
        self.unit_reg = unit_registry

    def learn_about_params(self, msg, length: quantity):
        # check the parameter is of the correct type
        assert length.check("[length]")

    def test_understanding_of_params(self):
        val_metres = quantity(33, self.unit_reg.metre)
        assert val_metres.check("[length]")
        self.learn_about_params("quantity", val_metres)

        val_yards = quantity(33, self.unit_reg.yard)
        assert val_yards.check("[length]")
        self.learn_about_params("quantity", val_yards)

    @unittest.expectedFailure
    def test_sending_wrong_dimension(self):
        bad_units = quantity(54, self.unit_reg.radian)

        # this should fail, since it's expecting a length
        self.learn_about_params("quantity", bad_units)

    @unittest.expectedFailure
    def test_try_to_send_non_quantity_to_quantity_method(self):
        self.learn_about_params("float", 46.0)

    def test_round_trip_degs(self):
        heading_degrees = quantity(180.0, self.unit_reg.degrees)
        self.assertEqual("<Quantity(180.0, 'degree')>", repr(heading_degrees))

        heading_radians = heading_degrees.to(self.unit_reg.radians)
        self.assertEqual(
            "<Quantity(3.141592653589793, 'radian')>", repr(heading_radians)
        )

        back_to_degree = heading_radians.to(self.unit_reg.degrees)
        self.assertEqual("<Quantity(180.0, 'degree')>", repr(back_to_degree))

    def test_roundTripSpeed(self):
        speed_kilometers = quantity(20, self.unit_reg.knot)
        self.assertEqual("<Quantity(20, 'knot')>", repr(speed_kilometers))

        speed_meters = speed_kilometers.to(self.unit_reg.meter / self.unit_reg.second)
        self.assertEqual(
            "<Quantity(10.28888888888889, 'meter / second')>", repr(speed_meters)
        )

        back_to_kilometes = speed_meters.to(self.unit_reg.knot)
        self.assertEqual(
            "<Quantity(20.000000000000004, 'knot')>", repr(back_to_kilometes)
        )
        self.assertAlmostEqual(20, back_to_kilometes.magnitude)

    def test_stateConversion(self):
        state = State(
            1,
            "100112 120800 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 ",
        )
        state.parse()

        # Speed and Heading from state
        # heading -> 109.08 (degrees)
        # speed   -> 6.00 (knots)

        self.assertEqual("<Quantity(109.08, 'degree')>", repr(state.get_heading()))
        self.assertEqual("<Quantity(6.0, 'knot')>", repr(state.get_speed()))


if __name__ == "__main__":
    unittest.main()



import sys
from .data_resolver import DataResolver
from .command_line_input import get_choice_input


class CommandLineResolver(DataResolver):
    def __init__(self):
        self.table_privacies = {}

    def synonym_search(self, data_store, platform_name):
        input_ = input("Please type word stem to search for: ")
        result = data_store.search_platform(input_)
        if result is None:
            # couldn't find it
            not_found = get_choice_input(
                f"Platform with '{input_}' not found. Do you wish to: ",
                [
                    "Search for another synonym of this name",
                    f"Add a new platform, titled '{platform_name}'",
                    "Cancel import",
                ],
            )

            if not_found == 1:
                self.synonym_search(data_store, platform_name)
            elif not_found == 2:
                return 2
            elif not_found == 3:
                print("Quitting")
                sys.exit(1)
        else:
            # found something
            found = get_choice_input(
                f"Platform '{result}' found. Would you like to add this as a synonym: ",
                ["Yes", "No, find other synonym", "Cancel import"],
            )
            if found == 1:
                return result
            elif found == 2:
                self.synonym_search(data_store, platform_name)
            elif found == 3:
                print("Quitting")
                sys.exit(1)

    def add_to_platforms(
        self, data_store, platform_name, platform_type_str, nationality_str, privacy_str
    ):
        print("Ok, adding new platform.")

        # Choose Nationality
        if nationality_str:
            chosen_nationality = data_store.add_to_nationalities(nationality_str)
        else:
            nationalities = data_store.get_nationalities()
            nationality_names = [n.name for n in nationalities]
            nationality_names.append("Add a new nationality")
            nationality_names.append("Cancel import")
            choice = get_choice_input("Please provide nationality: ", nationality_names)
            if choice == len(nationality_names):
                print("Quitting")
                sys.exit(1)
            elif choice == len(nationality_names) - 1:
                nationality_check_ok = False
                while not nationality_check_ok:
                    new_input = input("Please type name of new nationality: ")
                    nationality_check_ok = data_store.check_nationality(new_input)
                chosen_nationality = data_store.add_to_nationalities(new_input)

        # Choose Platform Type
        if platform_type_str:
            chosen_platform_type = data_store.add_to_platform_types(platform_type_str)
        else:
            platform_types = data_store.get_platform_types()
            platform_type_names = [c.name for c in platform_types]
            platform_type_names.append("Add a new class")
            platform_type_names.append("Cancel import")
            choice = get_choice_input("Ok, please provide class: ", platform_type_names)
            if choice == len(platform_type_names):
                print("Quitting")
                sys.exit(1)
            elif choice == len(platform_type_names) - 1:
                platform_type_ok = False
                while not platform_type_ok:
                    new_input = input("Please type name of new class: ")
                    platform_type_ok = data_store.check_platform_type(new_input)
                chosen_platform_type = data_store.add_to_platform_types(new_input)
            else:
                chosen_platform_type = platform_types[choice - 2]

        # Choose Sensor
        sensors = data_store.get_sensors_by_platform_type(chosen_platform_type)
        if len(sensors) > 0:
            sensor_names = [s.Sensor.name for s in sensors]
            sensor_names.append("Cancel import")
            choice = get_choice_input(
                f"We have {len(sensor_names) - 1} other instances of "
                f"{chosen_platform_type.name} class. They contain these "
                f"sensors. Please indicate which you wish to add to "
                f"{platform_name}: ",
                sensor_names,
            )
            if choice == len(sensor_names):
                print("Quitting")
                sys.exit(1)

            chosen_sensor = sensors[choice - 1]
        else:
            print("No sensors found for that class. Skipping Sensor add")
            chosen_sensor = None
            # TODO: Should we delete the code below?
            # choices = ["Add a new sensor", "Cancel import"]
            # choice = get_choice_input(f"No instances of class {chosen_platform_type.name} exist. Please choose an option: ",
            #                                                choices)
            # if choice == len(choices):
            #     print("Quitting")
            #     sys.exit(1)
            # elif choice == len(choices)-1:
            #     sensorCheckOk = False
            #     while not sensorCheckOk:
            #         newSensorInput = input("Please type name of new sensor: ")
            #         sensorCheckOk = data_store.checkSensor(newSensorInput)
            #     #chosen_sensor = data_store.addToSensors(new_input)
            #     chosen_sensor = newSensorInput
            #     newSensor = True

        # Choose Privacy
        if privacy_str:
            chosen_privacy = data_store.add_to_privacies(privacy_str)
        else:
            privacies = data_store.get_privacies()
            privacy_names = [c.name for c in privacies]
            privacy_names.append("Add a new classification")
            privacy_names.append("Cancel import")
            choice = get_choice_input(
                "Ok, please provide classification for this platform: ", privacy_names
            )

            if choice == len(privacy_names):
                print("Quitting")
                sys.exit(1)
            elif choice == len(privacy_names) - 1:
                privacy_check_ok = False
                while not privacy_check_ok:
                    new_input = input("Please type name of new classification: ")
                    privacy_check_ok = data_store.check_privacy(new_input)
                chosen_privacy = data_store.add_to_privacies(new_input)
            else:
                chosen_privacy = privacies[choice - 2]

        print("Input complete. About to create this platform:")
        print(f"Name: {platform_name}")
        print(f"Nationality: {chosen_nationality.name}")
        print(f"Class: {chosen_platform_type.name}")
        if chosen_sensor:
            print(f"Sensors: {chosen_sensor.Sensor.name}")
        else:
            print("Sensors: None")
        print(f"Classification: {chosen_privacy.name}")

        choice = get_choice_input(
            "Create this platform?: ",
            ["Yes", "No, make further edits", "Cancel import"],
        )

        if choice == 1:
            # TODO: pass back sensor and classification when Schema changed
            return (
                platform_name,
                chosen_platform_type,
                chosen_nationality,
                chosen_privacy,
            )
        elif choice == 2:
            return self.add_to_platforms(data_store, platform_name)
        elif choice == 3:
            print("Quitting")
            sys.exit(1)

    def resolve_platform(
        self, data_store, platform_name, platform_type, nationality, privacy
    ):
        choice = get_choice_input(
            f"Platform '{platform_name}' not found. Do you wish to: ",
            [  # "Search for synonym of this name",
                f"Add a new platform, titled '{platform_name}'",
                "Cancel import",
            ],
        )

        if choice == 1:
            # synSearch = self.synonym_search(data_store, platform_name)
            # print(f"Adding {synSearch} as a synonym for {platform_name}")
            return self.add_to_platforms(
                data_store, platform_name, platform_type, nationality, privacy
            )
        else:
            print("Quitting")
            sys.exit(1)

    def add_to_sensors(self, data_store, sensor_name):
        # Choose Sensor Type
        print("Ok, adding new sensor.")
        sensor_types = data_store.get_sensor_types()
        sensor_type_names = [st.name for st in sensor_types]
        sensor_type_names.append("Add a new sensor type")
        sensor_type_names.append("Cancel import")
        choice = get_choice_input("Please provide sensor type: ", sensor_type_names)
        if choice == len(sensor_type_names):
            print("Quitting")
            sys.exit(1)
        elif choice == len(sensor_type_names) - 1:
            sensor_type_check_ok = False
            while not sensor_type_check_ok:
                new_input = input("Please type name of new sensor type: ")
                sensor_type_check_ok = data_store.check_sensor_type(new_input)
            chosen_sensor_type = data_store.add_to_sensor_types(new_input)
        else:
            chosen_sensor_type = sensor_types[choice - 2]

        return sensor_name, chosen_sensor_type

    def resolve_sensor(self, data_store, sensor_name):
        choice = get_choice_input(
            f"Sensor '{sensor_name}' not found. Do you wish to: ",
            [f"Add a new sensor, titled '{sensor_name}'", "Cancel import"],
        )

        if choice == 1:
            return self.add_to_sensors(data_store, sensor_name)
        elif choice == 2:
            print("Quitting")
            sys.exit(1)

    def resolve_privacy(self, data_store, table_type_id, table_name):
        if table_type_id in self.table_privacies:
            return table_type_id, self.table_privacies[table_type_id]

        # Choose Privacy
        privacies = data_store.get_privacies()
        privacy_names = [c.name for c in privacies]
        privacy_names.append("Add a new classification")
        privacy_names.append("Cancel import")
        choice = get_choice_input(
            f"Ok, please provide classification for table '{table_name}': ",
            privacy_names,
        )

        if choice == len(privacy_names):
            print("Quitting")
            sys.exit(1)
        elif choice == len(privacy_names) - 1:
            privacy_check_ok = False
            while not privacy_check_ok:
                new_input = input("Please type name of new classification: ")
                privacy_check_ok = data_store.check_privacy(new_input)
            chosen_privacy = data_store.add_to_privacies(new_input)
        else:
            chosen_privacy = privacies[choice - 2]

        self.table_privacies[table_type_id] = chosen_privacy

        return table_type_id, chosen_privacy



import unittest
import datetime

from pepys_import.core.formats import unit_registry
from pepys_import.core.formats.state import State


class BasicTests(unittest.TestCase):
    def test_long_timestamp(self):
        # long date
        rep_line = State(
            1,
            "19951212 120800 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 ",
        )
        rep_line.parse()

        # long time
        rep_line = State(
            1,
            "951212 120800.555 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 ",
        )
        rep_line.parse()

    def test_Getters(self):
        rep_line = State(
            1,
            "951212 120800.555 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 ",
        )
        rep_line.parse()

        rep_line.get_line_number
        rep_line.get_timestamp
        rep_line.get_platform
        rep_line.get_symbology
        rep_line.get_latitude
        rep_line.get_longitude
        rep_line.get_heading
        rep_line.get_speed
        rep_line.get_depth
        rep_line.get_text_label

    def test_error_reports(self):
        # too few fields
        rep_line = State(1, " 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 ")
        rep_line.parse()

        # wrong length date
        rep_line = State(
            1, "12 120800 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 "
        )
        rep_line.parse()

        # wrong length time
        rep_line = State(
            1, "951212 12 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 "
        )
        rep_line.parse()

        # wrong length symbology
        rep_line = State(
            1,
            "951212 120800 SUBJECT VCC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 ",
        )
        rep_line.parse()

        # bad latitude
        rep_line = State(
            1,
            "951212 120800 SUBJECT VC 6A 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 ",
        )
        rep_line.parse()

        # bad longtude
        rep_line = State(
            1,
            "951212 120800 SUBJECT VC 60 23 40.25 N 00A 01 2B.86 E 109.08  6.00  0.00 ",
        )
        rep_line.parse()

        # bad course
        rep_line = State(
            1,
            "951212 120800 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 10a.08  6.00  0.00 ",
        )
        rep_line.parse()

        # bad speed
        rep_line = State(
            1,
            "951212 120800 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.b0  0.00 ",
        )
        rep_line.parse()

        # bad depth
        rep_line = State(
            1,
            "951212 120800 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.c0 ",
        )
        rep_line.parse()

    def test_line_ok(self):
        rep_line = State(
            1,
            "100112 120800 SUBJECT VC 60 23 40.25 N 000 01 25.86 E 109.08  6.00  0.00 ",
        )
        rep_line.parse()

        self.assertEqual(1, rep_line.get_line_number())

        print(rep_line.get_timestamp())

        uReg = unit_registry

        self.assertEqual(
            datetime.datetime(2010, 1, 12, 12, 8), rep_line.get_timestamp()
        )
        self.assertEqual("SUBJECT", rep_line.get_platform())
        self.assertEqual("VC", rep_line.get_symbology())
        # FixMe   self.assertEqual(Location(60.0, 23.0, 40.25, "N"), rep_line.getLatitude())
        # FixMe   self.assertEqual("SUBJECT", rep_line.getLongitude())
        self.assertAlmostEqual(1.9038051480754146, rep_line.get_heading())
        self.assertAlmostEqual(6.00, rep_line.get_speed().to(uReg.knot).magnitude)
        self.assertEqual(0.0, rep_line.get_depth())
        self.assertEqual(None, rep_line.get_text_label())


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from unittest import TestCase
from sqlalchemy import inspect

from pepys_import.core.store.data_store import DataStore
from pepys_import.core.formats.repl_file import REPFile
from pepys_import.resolvers.command_line_resolver import CommandLineResolver

FILE_PATH = os.path.dirname(__file__)
TEST_DATA_PATH = os.path.join(FILE_PATH, "sample_data", "rep_files")
TEST_FILE = os.path.join(TEST_DATA_PATH, "rep_test1.rep")
BROKEN_FILE = os.path.join(TEST_DATA_PATH, "rep_test2.rep")


class TestLoadReplay(TestCase):
    @unittest.skip("Skip until we can automate command-line-resolver")
    def test_load_replay(self):
        """Test  whether we can load REP data"""
        data_store = DataStore(
            "",
            "",
            "",
            0,
            ":memory:",
            db_type="sqlite",
            missing_data_resolver=CommandLineResolver(),
        )

        # creating database from schema
        data_store.initialise()

        rep_file = REPFile(TEST_FILE)
        self.assertEqual("REP", rep_file.get_data_file_type())

        with data_store.session_scope() as session:
            datafile = session.add_to_datafile_from_rep(
                rep_file.get_data_file_name(), rep_file.get_data_file_type()
            )
            for repLine in rep_file.get_lines():
                platform = session.add_to_platforms_from_rep(
                    repLine.get_platform(), "Fisher", "UK", "Public"
                )
                sensor = session.add_to_sensors_from_rep("GPS", platform)
                session.add_to_states_from_rep(
                    repLine.get_timestamp(),
                    datafile,
                    sensor,
                    repLine.get_latitude(),
                    repLine.get_longitude(),
                    repLine.get_heading(),
                    repLine.get_speed(),
                )

        inspector = inspect(data_store.engine)
        table_names = inspector.get_table_names()

        # 11 tables must be created. A few of them tested
        self.assertEqual(len(table_names), 11)
        self.assertIn("Entry", table_names)
        self.assertIn("Platforms", table_names)
        self.assertIn("State", table_names)
        self.assertIn("Datafiles", table_names)
        self.assertIn("Nationalities", table_names)


if __name__ == "__main__":
    unittest.main()



from pyfiglet import Figlet


def render_text(font, text):
    f = Figlet(font=font)
    print(f.renderText(text))


def show_welcome_banner():
    font = "doom"
    text = "Pepys_import"
    render_text(font, text)


def show_software_meta_info(version, db_type, db_name, db_host):
    print("Software Version : ", version, "\n\n")
    print("Database Type : ", db_type)
    print("Database Name : ", db_name)
    print("Database Host : ", db_host)



import os
from pathlib import Path

from datetime import datetime
from sqlalchemy import create_engine, event
from sqlalchemy.schema import CreateSchema
from sqlalchemy.orm import sessionmaker
from sqlalchemy.exc import OperationalError
from importlib import import_module
from contextlib import contextmanager

from pepys_import.resolvers.default_resolver import DefaultResolver
from pepys_import.utils.data_store_utils import import_from_csv
from .db_base import base_postgres, base_sqlite
from .db_status import TableTypes
from pepys_import.core.formats import unit_registry

from pepys_import import __version__
from pepys_import.utils.branding_util import (
    show_welcome_banner,
    show_software_meta_info,
)

MAIN_DIRECTORY_PATH = Path(__file__).parent.parent.parent  # pepys_import/pepys_import
DEFAULT_DATA_PATH = os.path.join(MAIN_DIRECTORY_PATH, "database", "default_data")

# TODO: add foreign key refs
# TODO: add proper uuid funcs that interact with entries table

# TODO: probably move this module to the top level as it's the main one


class DataStore:

    # TODO: supply or lookup user id
    # Valid options for db_type are 'postgres' and 'sqlite'
    def __init__(
        self,
        db_username,
        db_password,
        db_host,
        db_port,
        db_name,
        db_type="postgres",
        missing_data_resolver=DefaultResolver(),
        show_welcome=True,
        show_status=True,
    ):
        if db_type == "postgres":
            self.db_classes = import_module("pepys_import.core.store.postgres_db")
            driver = "postgresql+psycopg2"
        elif db_type == "sqlite":
            self.db_classes = import_module("pepys_import.core.store.sqlite_db")
            driver = "sqlite+pysqlite"
        else:
            raise Exception(
                f"Unknown db_type {db_type} supplied, if specified should be one of 'postgres' or 'sqlite'"
            )

        # setup meta_class data
        self.meta_classes = {}
        self.setup_table_type_mapping()

        connection_string = "{}://{}:{}@{}:{}/{}".format(
            driver, db_username, db_password, db_host, db_port, db_name
        )
        self.engine = create_engine(connection_string, echo=False)

        if db_type == "postgres":
            base_postgres.metadata.bind = self.engine
        elif db_type == "sqlite":
            base_sqlite.metadata.bind = self.engine

        self.missing_data_resolver = missing_data_resolver
        self.show_welcome = show_welcome
        self.show_status = show_status

        # caches of known data
        self.table_types = {}
        self.privacies = {}
        self.nationalities = {}
        self.datafile_types = {}
        self.datafiles = {}
        self.platform_types = {}
        self.platforms = {}
        self.sensor_types = {}
        self.sensors = {}

        # TEMP list of values for defaulted IDs, to be replaced by missing info lookup mechanism
        self.default_user_id = 1  # DevUser

        # Instance attributes which are necessary for initialise method
        self.db_name = db_name
        self.db_type = db_type

        # use session_scope() to create a new session
        self.session = None

        # Branding Text
        if self.show_welcome:
            show_welcome_banner()
        if self.show_status:
            show_software_meta_info(__version__, self.db_type, self.db_name, db_host)
            print("---------------------------------")

    def initialise(self):
        """Create schemas for the database"""

        if self.db_type == "sqlite":
            try:
                # Attempt to create schema if not present, to cope with fresh DB file
                base_sqlite.metadata.create_all(self.engine)
            except OperationalError:
                print(
                    "Error creating database schema, possible invalid path? ('"
                    + self.db_name
                    + "'). Quitting"
                )
                exit()
        elif self.db_type == "postgres":
            try:
                #  ensure that create schema scripts created before create table scripts
                event.listen(
                    base_postgres.metadata,
                    "before_create",
                    CreateSchema("datastore_schema"),
                )
                base_postgres.metadata.create_all(self.engine)
            except OperationalError:
                print(f"Error creating database({self.db_name})! Quitting")
                exit()

    @contextmanager
    def session_scope(self):
        """Provide a transactional scope around a series of operations."""
        db_session = sessionmaker(bind=self.engine)
        self.session = db_session()
        try:
            yield self
            self.session.commit()
        except:
            self.session.rollback()
            raise
        finally:
            self.session.close()

    #############################################################
    # Add functions

    def add_to_table_types(self, table_type_id, table_name):
        # check in cache for table type
        if table_type_id in self.table_types:
            return self.table_types[table_type_id]

        # doesn't exist in cache, try to lookup in DB
        table_types = self.search_table_type(table_type_id)
        if table_types:
            # add to cache and return
            self.table_types[table_type_id] = table_types
            return table_types

        # enough info to proceed and create entry
        table_type = self.db_classes.TableType(
            table_type_id=table_type_id, name=table_name
        )
        self.session.add(table_type)
        self.session.flush()

        # add to cache and return created table type
        self.table_types[table_type_id] = table_type
        # should return DB type or something else decoupled from DB?
        return table_type

    def add_to_entries(self, table_type_id, tableName):
        # ensure table type exists to satisfy foreign key constraint
        self.add_to_table_types(table_type_id, tableName)

        # No cache for entries, just add new one when called
        entry_obj = self.db_classes.Entry(
            table_type_id=table_type_id, created_user=self.default_user_id
        )

        self.session.add(entry_obj)
        self.session.flush()

        return entry_obj.entry_id

    # TODO: add function to do common pattern of action in these functions
    def add_to_platform_types(self, platform_type_name):
        # check in cache for nationality
        if platform_type_name in self.platform_types:
            return self.platform_types[platform_type_name]

        # doesn't exist in cache, try to lookup in DB
        platform_types = self.search_platform_type(platform_type_name)
        if platform_types:
            # add to cache and return looked up platform type
            self.platform_types[platform_type_name] = platform_types
            return platform_types

        # enough info to proceed and create entry
        platform_type = self.db_classes.PlatformType(name=platform_type_name)
        self.session.add(platform_type)
        self.session.flush()

        # add to cache and return created platform type
        self.platform_types[platform_type_name] = platform_type
        # should return DB type or something else decoupled from DB?
        return platform_type

    def add_to_nationalities(self, nationality_name):
        # check in cache for nationality
        if nationality_name in self.nationalities:
            return self.nationalities[nationality_name]

        # doesn't exist in cache, try to lookup in DB
        nationalities = self.search_nationality(nationality_name)
        if nationalities:
            # add to cache and return looked up nationality
            self.nationalities[nationality_name] = nationalities
            return nationalities

        # enough info to proceed and create entry
        nationality = self.db_classes.Nationality(name=nationality_name)
        self.session.add(nationality)
        self.session.flush()

        # add to cache and return created platform
        self.nationalities[nationality_name] = nationality
        # should return DB type or something else decoupled from DB?
        return nationality

    def add_to_privacies(self, privacy_name):
        # check in cache for privacy
        if privacy_name in self.privacies:
            return self.privacies[privacy_name]

        # doesn't exist in cache, try to lookup in DB
        privacies = self.search_privacy(privacy_name)
        if privacies:
            # add to cache and return looked up platform
            self.privacies[privacy_name] = privacies
            return privacies

        # enough info to proceed and create entry
        privacy = self.db_classes.Privacy(name=privacy_name)
        self.session.add(privacy)
        self.session.flush()

        # add to cache and return created platform
        self.privacies[privacy_name] = privacy
        # should return DB type or something else decoupled from DB?
        return privacy

    # TODO: it is possible to merge two methods taking a resolver=True/False argument
    def add_to_datafile_types(self, datafile_type):
        # check in cache for datafile type
        if datafile_type in self.datafile_types:
            return self.datafile_types[datafile_type]

        # doesn't exist in cache, try to lookup in DB
        datafile_types = self.search_datafile_type(datafile_type)
        if datafile_types:
            # add to cache and return looked up datafile type
            self.datafile_types[datafile_type] = datafile_types
            return datafile_types

        # proceed and create entry
        datafile_type_obj = self.db_classes.DatafileType(name=datafile_type)

        self.session.add(datafile_type_obj)
        self.session.flush()

        # add to cache and return created datafile type
        self.datafile_types[datafile_type] = datafile_type_obj
        # should return DB type or something else decoupled from DB?
        return datafile_type_obj

    def add_to_datafile_from_rep(self, datafile_name, datafile_type):
        # check in cache for datafile
        if datafile_name in self.datafiles:
            return self.datafiles[datafile_name]

        # doesn't exist in cache, try to lookup in DB
        datafiles = self.search_datafile(datafile_name)
        if datafiles:
            # add to cache and return looked up datafile
            self.datafiles[datafile_name] = datafiles
            return datafiles

        datafile_type_obj = self.add_to_datafile_types(datafile_type)

        # don't know privacy, use resolver to query for data
        privacy = self.missing_data_resolver.resolve_privacy(
            self,
            self.db_classes.Datafile.table_type_id,
            self.db_classes.Datafile.__tablename__,
        )

        # privacy should contain (tabletype, privacy_name)
        # enough info to proceed and create entry
        table_type, privacy = privacy
        entry_id = self.add_to_entries(
            self.db_classes.Datafile.table_type_id,
            self.db_classes.Datafile.__tablename__,
        )

        datafile_obj = self.db_classes.Datafile(
            datafile_id=entry_id,
            simulated=False,
            reference=datafile_name,
            url=None,
            privacy_id=privacy.privacy_id,
            datafile_type_id=datafile_type_obj.datafile_type_id,
        )

        self.session.add(datafile_obj)
        self.session.flush()

        self.datafiles[datafile_name] = datafile_obj
        # should return DB type or something else decoupled from DB?
        return datafile_obj

    def add_to_platforms_from_rep(
        self, platform_name, platform_type, nationality, privacy
    ):
        # check in cache for platform
        if platform_name in self.platforms:
            return self.platforms[platform_name]

        # doesn't exist in cache, try to lookup in DB
        platforms = self.search_platform(platform_name)
        if platforms:
            # add to cache and return looked up platform
            self.platforms[platform_name] = platforms
            return platforms

        # doesn't exist in DB, use resolver to query for data
        platform = self.missing_data_resolver.resolve_platform(
            self, platform_name, platform_type, nationality, privacy
        )

        # platform should contain (platform_name, platform_type, nationality)
        # enough info to proceed and create entry
        _, platform_type, nationality, privacy = platform
        entry_id = self.add_to_entries(
            self.db_classes.Platform.table_type_id,
            self.db_classes.Platform.__tablename__,
        )

        platform_obj = self.db_classes.Platform(
            platform_id=entry_id,
            name=platform_name,
            platform_type_id=platform_type.platform_type_id,
            host_platform_id=None,
            nationality_id=nationality.nationality_id,
            privacy_id=privacy.privacy_id,
        )

        self.session.add(platform_obj)
        self.session.flush()

        # add to cache and return created platform
        self.platforms[platform_name] = platform_obj
        # should return DB type or something else decoupled from DB?
        return platform_obj

    def add_to_sensor_types(self, sensor_type_name):
        # check in cache for sensor type
        if sensor_type_name in self.sensor_types:
            return self.sensor_types[sensor_type_name]

        # doesn't exist in cache, try to lookup in DB
        sensor_types = self.search_sensor_type(sensor_type_name)
        if sensor_types:
            # add to cache and return looked up sensor type
            self.sensor_types[sensor_type_name] = sensor_types
            return sensor_types

        # enough info to proceed and create entry
        sensor_type = self.db_classes.SensorType(name=sensor_type_name)
        self.session.add(sensor_type)
        self.session.flush()

        # add to cache and return created sensor type
        self.sensor_types[sensor_type_name] = sensor_type
        # should return DB type or something else decoupled from DB?
        return sensor_type

    def add_to_sensors_from_rep(self, sensor_name, platform):
        # check in cache for sensor
        if sensor_name in self.sensors:
            return self.sensors[sensor_name]

        # doesn't exist in cache, try to lookup in DB
        sensors = self.search_sensor(sensor_name)
        if sensors:
            # add to cache and return looked up sensor
            self.sensors[sensor_name] = sensors
            return sensors

        # doesn't exist in DB, use resolver to query for data
        sensor = self.missing_data_resolver.resolve_sensor(self, sensor_name)

        # sensor should contain (sensor_name, sensorType)
        # enough info to proceed and create entry
        _, sensor_type = sensor
        entry_id = self.add_to_entries(
            self.db_classes.Sensor.table_type_id, self.db_classes.Sensor.__tablename__
        )

        sensor_obj = self.db_classes.Sensor(
            sensor_id=entry_id,
            name=sensor_name,
            sensor_type_id=self.db_classes.map_uuid_type(sensor_type.sensor_type_id),
            platform_id=self.db_classes.map_uuid_type(platform.platform_id),
        )

        self.session.add(sensor_obj)
        self.session.flush()

        # add to cache and return created sensor
        self.sensors[sensor_name] = sensor_obj
        # should return DB type or something else decoupled from DB?
        return sensor_obj

    def add_to_states_from_rep(
        self, timestamp, datafile, sensor, lat, long, heading, speed
    ):
        # No cache for entries, just add new one when called

        # don't know privacy, use resolver to query for data
        privacy = self.missing_data_resolver.resolve_privacy(
            self,
            self.db_classes.State.table_type_id,
            self.db_classes.State.__tablename__,
        )

        # privacy should contain (table_type, privacy_name)
        # enough info to proceed and create entry
        table_type, privacy = privacy
        entry_id = self.add_to_entries(
            self.db_classes.State.table_type_id, self.db_classes.State.__tablename__
        )

        # heading is a quantity. Convert to radians
        heading_rads = heading.to(unit_registry.radians).magnitude

        # speed is a quantity. Convert to m/sec
        speed_m_sec = speed.to(unit_registry.meter / unit_registry.second).magnitude

        state_obj = self.db_classes.State(
            state_id=entry_id,
            time=timestamp,
            sensor_id=sensor.sensor_id,
            location="(" + str(long.degrees) + "," + str(lat.degrees) + ")",
            heading=heading_rads,
            speed=speed_m_sec,
            datafile_id=datafile.datafile_id,
            privacy_id=privacy.privacy_id,
        )
        self.session.add(state_obj)
        self.session.flush()

        return state_obj

    def add_to_states(
        self,
        time,
        sensor,
        datafile,
        location=None,
        heading=None,
        course=None,
        speed=None,
        privacy=None,
    ):
        time = datetime.strptime(time, "%Y-%m-%d %H:%M:%S")

        sensor = self.search_sensor(sensor)
        datafile = self.search_datafile(datafile)
        privacy = self.search_privacy(privacy)

        if sensor is None or datafile is None:
            print(f"There is missing value(s) in '{sensor}, {datafile}'!")
            return

        heading_rads = None
        if heading:
            # heading is a string, turn into quantity. Convert to radians
            heading_quantity = float(heading) * unit_registry.knot
            heading_rads = heading_quantity.to(unit_registry.radians).magnitude

        speed_m_sec = None
        if speed:
            # speed is a string, turn into quantity. Convert to m/sec
            speed_quantity = float(speed) * unit_registry.knot
            speed_m_sec = speed_quantity.to(
                unit_registry.meter / unit_registry.second
            ).magnitude

        entry_id = self.add_to_entries(
            self.db_classes.State.table_type_id, self.db_classes.State.__tablename__
        )
        state_obj = self.db_classes.State(
            state_id=entry_id,
            time=time,
            sensor_id=sensor.sensor_id,
            location=location,
            heading=heading_rads,
            # course=course,
            speed=speed_m_sec,
            datafile_id=datafile.datafile_id,
            privacy_id=privacy.privacy_id,
        )
        self.session.add(state_obj)
        self.session.flush()

        return state_obj

    def add_to_sensors(self, name, sensor_type, host):
        sensor_type = self.search_sensor_type(sensor_type)
        host = self.search_platform(host)

        if sensor_type is None or host is None:
            print(f"There is missing value(s) in '{sensor_type}, {host}'!")
            return

        entry_id = self.add_to_entries(
            self.db_classes.Sensor.table_type_id, self.db_classes.Sensor.__tablename__
        )

        sensor_obj = self.db_classes.Sensor(
            sensor_id=entry_id,
            name=name,
            sensor_type_id=sensor_type.sensor_type_id,
            platform_id=host.platform_id,
        )
        self.session.add(sensor_obj)
        self.session.flush()

        return sensor_obj

    def add_to_datafiles(self, simulated, privacy, file_type, reference=None, url=None):

        privacy = self.search_privacy(privacy)
        datafile_type = self.search_datafile_type(file_type)

        if privacy is None or datafile_type is None:
            print("There is missing value(s) in the data!")
            return

        entry_id = self.add_to_entries(
            self.db_classes.Datafile.table_type_id,
            self.db_classes.Datafile.__tablename__,
        )

        datafile_obj = self.db_classes.Datafile(
            datafile_id=entry_id,
            simulated=bool(simulated),
            privacy_id=privacy.privacy_id,
            datafile_type_id=datafile_type.datafile_type_id,
            reference=reference,
            url=url,
        )

        self.session.add(datafile_obj)
        self.session.flush()

        return datafile_obj

    def add_to_platforms(self, name, nationality, platform_type, privacy):

        nationality = self.search_nationality(nationality)
        platform_type = self.search_platform_type(platform_type)
        privacy = self.search_privacy(privacy)

        if nationality is None or platform_type is None or privacy is None:
            print("There is missing value(s) in the data!")
            return

        entry_id = self.add_to_entries(
            self.db_classes.Platform.table_type_id,
            self.db_classes.Platform.__tablename__,
        )

        platform_obj = self.db_classes.Platform(
            platform_id=entry_id,
            name=name,
            nationality_id=nationality.nationality_id,
            platform_type_id=platform_type.platform_type_id,
            privacy_id=privacy.privacy_id,
        )

        self.session.add(platform_obj)
        self.session.flush()

        # add to cache and return created platform
        self.platforms[name] = platform_obj
        # should return DB type or something else decoupled from DB?
        return platform_obj

    #############################################################
    # Search/lookup functions

    def search_datafile_type(self, name):
        # search for any datafile type with this name
        return (
            self.session.query(self.db_classes.DatafileType)
            .filter(self.db_classes.DatafileType.name == name)
            .first()
        )

    def search_datafile(self, name):
        # search for any datafile with this name
        return (
            self.session.query(self.db_classes.Datafile)
            .filter(self.db_classes.Datafile.reference == name)
            .first()
        )

    def search_platform(self, name):
        # search for any platform with this name
        return (
            self.session.query(self.db_classes.Platform)
            .filter(self.db_classes.Platform.name == name)
            .first()
        )

    def search_platform_type(self, name):
        # search for any platform type with this name
        return (
            self.session.query(self.db_classes.PlatformType)
            .filter(self.db_classes.PlatformType.name == name)
            .first()
        )

    def search_nationality(self, name):
        # search for any nationality with this name
        return (
            self.session.query(self.db_classes.Nationality)
            .filter(self.db_classes.Nationality.name == name)
            .first()
        )

    def search_sensor(self, name):
        # search for any sensor type featuring this name
        return (
            self.session.query(self.db_classes.Sensor)
            .filter(self.db_classes.Sensor.name == name)
            .first()
        )

    def search_sensor_type(self, name):
        # search for any sensor type featuring this name
        return (
            self.session.query(self.db_classes.SensorType)
            .filter(self.db_classes.SensorType.name == name)
            .first()
        )

    def search_privacy(self, name):
        # search for any privacy with this name
        return (
            self.session.query(self.db_classes.Privacy)
            .filter(self.db_classes.Privacy.name == name)
            .first()
        )

    def search_table_type(self, table_type_id):
        # search for any table type with this id
        return (
            self.session.query(self.db_classes.TableType)
            .filter(self.db_classes.TableType.table_type_id == table_type_id)
            .first()
        )

    #############################################################
    # Get functions

    def get_datafiles(self):
        # get list of all datafiles in the DB
        return self.session.query(self.db_classes.Datafile).all()

    def get_nationalities(self):
        # get list of all nationalities in the DB
        return self.session.query(self.db_classes.Nationality).all()

    def get_platforms(self):
        # get list of all platforms in the DB
        return self.session.query(self.db_classes.Platform).all()

    def get_platform_types(self):
        # get list of all platform types in the DB
        return self.session.query(self.db_classes.PlatformType).all()

    def get_privacies(self):
        # get list of all privacies in the DB
        return self.session.query(self.db_classes.Privacy).all()

    def get_sensors(self):
        # get list of all sensors in the DB
        return self.session.query(self.db_classes.Sensor).all()

    def get_sensor_types(self):
        # get list of all sensors types in the DB
        return self.session.query(self.db_classes.SensorType).all()

    def get_sensors_by_platform_type(self, platform_type):
        # given platform type, return all Sensors contained on platforms of that type
        return (
            self.session.query(self.db_classes.Platform, self.db_classes.Sensor)
            .join(
                self.db_classes.Sensor,
                self.db_classes.Sensor.platform_id
                == self.db_classes.Platform.platform_id,
            )
            .filter(
                self.db_classes.Platform.platform_type_id
                == platform_type.platform_type_id
            )
            .all()
        )

    def get_states(self):
        # get list of all states in the DB
        return self.session.query(self.db_classes.State).all()

    #############################################################
    # Validation/check functions

    # return True if provided nationality ok
    def check_nationality(self, nationality):
        if len(nationality) == 0:
            return False

        if next(
            (nat for nat in self.get_nationalities() if nat.name == nationality), None
        ):
            # A nationality already exists with that name
            return False

        return True

    # return True if provided privacy ok
    def check_privacy(self, privacy):
        if len(privacy) == 0:
            return False

        if next((priv for priv in self.get_privacies() if priv.name == privacy), None):
            # A privacy already exists with that name
            return False

        return True

    # return True if provided platform type ok
    def check_platform_type(self, platform_type):
        if len(platform_type) == 0:
            return False

        if next(
            (pt for pt in self.get_platform_types() if pt.name == platform_type), None
        ):
            # A platform type already exists with that name
            return False

        return True

    # return True if provided sensor ok
    def check_sensor(self, sensor):
        if len(sensor) == 0:
            return False

        if next((sen for sen in self.get_sensors() if sen.name == sensor), None):
            # A sensor already exists with that name
            return False

        return True

    # return True if provided sensor type ok
    def check_sensor_type(self, sensor_type):
        if len(sensor_type) == 0:
            return False

        if next((st for st in self.get_sensor_types() if st.name == sensor_type), None):
            # A sensor type already exists with that name
            return False

        return True

    #############################################################
    # Generic Metadata functions

    def setup_table_type_mapping(self):
        # setup a map of tables keyed by TableType
        db_classes = dict(
            [
                (name, cls)
                for name, cls in self.db_classes.__dict__.items()
                if isinstance(cls, type)
                and (issubclass(cls, base_postgres) or issubclass(cls, base_sqlite))
                and cls.__name__ != "Base"
            ]
        )
        for table_type in TableTypes:
            self.meta_classes[table_type] = [
                cls
                for name, cls in db_classes.items()
                if db_classes[name].table_type == table_type
            ]

    def get_table_type_data(self, table_types):
        data = dict()
        for table_type in table_types:
            for table in self.meta_classes[table_type]:
                data[table.__name__] = self.session.query(table).count()
        return data

    #############################################################
    # Populate methods in order to import CSV files

    def populate_reference(self, reference_data_folder=None):
        """Import given CSV file to the given reference table"""
        if reference_data_folder is None:
            reference_data_folder = DEFAULT_DATA_PATH

        files = os.listdir(reference_data_folder)

        reference_tables = []
        # Create reference table list
        reference_table_objects = self.meta_classes[TableTypes.REFERENCE]
        for table_object in list(reference_table_objects):
            reference_tables.append(table_object.__tablename__)

        reference_files = [
            file
            for file in files
            if os.path.splitext(file)[0].replace(" ", "") in reference_tables
        ]
        import_from_csv(self, reference_data_folder, reference_files)

    def populate_metadata(self, sample_data_folder=None):
        """Import CSV files from the given folder to the related Metadata Tables"""
        if sample_data_folder is None:
            sample_data_folder = DEFAULT_DATA_PATH

        files = os.listdir(sample_data_folder)

        metadata_tables = []
        # Create metadata table list
        metadata_table_objects = self.meta_classes[TableTypes.METADATA]
        for table_object in list(metadata_table_objects):
            metadata_tables.append(table_object.__tablename__)

        metadata_files = [
            file for file in files if os.path.splitext(file)[0] in metadata_tables
        ]
        import_from_csv(self, sample_data_folder, metadata_files)

    def populate_measurement(self, sample_data_folder=None):
        """Import CSV files from the given folder to the related Measurement Tables"""
        if sample_data_folder is None:
            sample_data_folder = DEFAULT_DATA_PATH

        files = os.listdir(sample_data_folder)

        measurement_tables = []
        # Create measurement table list
        measurement_table_objects = self.meta_classes[TableTypes.MEASUREMENT]
        for table_object in list(measurement_table_objects):
            measurement_tables.append(table_object.__tablename__)

        measurement_files = [
            file for file in files if os.path.splitext(file)[0] in measurement_tables
        ]
        import_from_csv(self, sample_data_folder, measurement_files)



import unittest

from pepys_import.core.store.data_store import DataStore
from testing.postgresql import Postgresql
from sqlalchemy import inspect
from unittest import TestCase


class TestDataStoreInitialisePostgres(TestCase):
    def setUp(self):
        self.store = None
        try:
            self.store = Postgresql(
                database="test", host="localhost", user="postgres", port=55527
            )
        except RuntimeError:
            print("PostgreSQL database couldn't be created! Test is skipping.")

    def tearDown(self):
        try:
            self.store.stop()
        except AttributeError:
            return

    def test_postgres_initialise(self):
        """Test whether schemas created successfully on PostgresSQL"""
        if self.store is None:
            self.skipTest("Postgres is not available. Test is skipping")

        data_store_postgres = DataStore(
            db_username="postgres",
            db_password="",
            db_host="localhost",
            db_port=55527,
            db_name="test",
        )

        # inspector makes it possible to load lists of schema, table, column
        # information for the sqlalchemy engine
        inspector = inspect(data_store_postgres.engine)
        table_names = inspector.get_table_names()
        schema_names = inspector.get_schema_names()

        # there must be no table, and no schema for datastore at the beginning
        self.assertEqual(len(table_names), 0)
        self.assertNotIn("datastore_schema", schema_names)

        # creating database from schema
        data_store_postgres.initialise()

        inspector = inspect(data_store_postgres.engine)
        table_names = inspector.get_table_names()
        schema_names = inspector.get_schema_names()

        # 11 tables must be created. A few of them tested
        self.assertEqual(len(table_names), 11)
        self.assertIn("Entry", table_names)
        self.assertIn("Platforms", table_names)
        self.assertIn("States", table_names)
        self.assertIn("Datafiles", table_names)
        self.assertIn("Nationalities", table_names)

        # datastore_schema must be created
        self.assertIn("datastore_schema", schema_names)


class TestDataStoreInitialiseSQLite(TestCase):
    def test_sqlite_initialise(self):
        """Test whether schemas created successfully on SQLite"""
        data_store_sqlite = DataStore("", "", "", 0, ":memory:", db_type="sqlite")

        # inspector makes it possible to load lists of schema, table, column
        # information for the sqlalchemy engine
        inspector = inspect(data_store_sqlite.engine)
        table_names = inspector.get_table_names()

        # there must be no table at the beginning
        self.assertEqual(len(table_names), 0)

        # creating database from schema
        data_store_sqlite.initialise()

        inspector = inspect(data_store_sqlite.engine)
        table_names = inspector.get_table_names()

        # 11 tables must be created. A few of them tested
        self.assertEqual(len(table_names), 11)
        self.assertIn("Entry", table_names)
        self.assertIn("Platforms", table_names)
        self.assertIn("States", table_names)
        self.assertIn("Datafiles", table_names)
        self.assertIn("Nationalities", table_names)


if __name__ == "__main__":
    unittest.main()



from datetime import datetime
from .location import Location
from . import unit_registry, quantity


class State:
    def __init__(self, line_number, line):
        self.line_num = line_number
        self.line = line

        self.timestamp = None
        self.vessel = None
        self.symbology = None
        self.latitude = None
        self.longitude = None
        self.heading = None
        self.speed = None
        self.depth = None
        self.text_label = None

        # Initialize pint's unit registry object
        self.unit_registry = unit_registry

    def print(self):
        print(
            "REP Line {} - Timestamp: {} Vessel: {} Symbology: {} Latitude: {} Longitude: {} Heading: {} Speed: {} Depth: {} TextLabel: {}".format(
                self.line_num,
                self.timestamp,
                self.vessel,
                self.symbology,
                self.latitude,
                self.longitude,
                self.heading,
                self.speed,
                self.depth,
                self.text_label,
            )
        )

    def parse(self):

        tokens = self.line.split()

        if len(tokens) < 15:
            print(
                "Error on line {} not enough tokens: {}".format(
                    self.line_num, self.line
                )
            )
            return False

        # separate token strings
        date_token = tokens[0]
        time_token = tokens[1]
        vessel_name_token = tokens[2]
        symbology_token = tokens[3]
        lat_degrees_token = tokens[4]
        lat_mins_token = tokens[5]
        lat_secs_token = tokens[6]
        lat_hemi_token = tokens[7]
        long_degrees_token = tokens[8]
        long_mins_token = tokens[9]
        long_secs_token = tokens[10]
        long_hemi_token = tokens[11]
        heading_token = tokens[12]
        speed_token = tokens[13]
        depth_token = tokens[14]
        text_label_token = ""

        if len(tokens) >= 16:
            # TODO: join back into single string, or extract full substring
            self.text_label = tokens[15:]

        if len(date_token) != 6 and len(date_token) != 8:
            print(
                "Line {}. Error in Date format {}. Should be either 2 of 4 figure date, followed by month then date".format(
                    self.line_num, date_token
                )
            )
            return False

        # Times always in Zulu/GMT
        if len(time_token) != 6 and len(time_token) != 10:
            print(
                "Line {}. Error in Time format {}. Should be HHMMSS[.SSS]".format(
                    self.line_num, time_token
                )
            )
            return False

        self.timestamp = self.parse_timestamp(date_token, time_token)

        self.vessel = vessel_name_token.strip('"')

        symbology_values = symbology_token.split("[")
        if len(symbology_values) >= 1:
            if len(symbology_values[0]) != 2 and len(symbology_values[0]) != 5:
                print(
                    "Line {}. Error in Symbology format {}. Should be 2 or 5 chars".format(
                        self.line_num, symbology_token
                    )
                )
                return False
        if len(symbology_values) != 1 and len(symbology_values) != 2:
            print(
                "Line {}. Error in Symbology format {}".format(
                    self.line_num, symbology_token
                )
            )
            return False

        self.symbology = symbology_token

        self.latitude = Location(
            lat_degrees_token, lat_mins_token, lat_secs_token, lat_hemi_token
        )
        if not self.latitude.parse():
            print("Line {}. Error in latitude parsing".format(self.line_num))
            return False

        self.longitude = Location(
            long_degrees_token, long_mins_token, long_secs_token, long_hemi_token
        )
        if not self.latitude.parse():
            print("Line {}. Error in longitude parsing".format(self.line_num))
            return False

        try:
            valid_heading = float(heading_token)
        except ValueError:
            print(
                "Line {}. Error in heading value {}. Couldn't convert to a number".format(
                    self.line_num, heading_token
                )
            )
            return False
        if 0.0 > valid_heading >= 360.0:
            print(
                "Line {}. Error in heading value {}. Should be be between 0 and 359.9 degrees".format(
                    self.line_num, heading_token
                )
            )
            return False

        # Set heading as degree(quantity-with-unit) object
        self.set_heading(valid_heading * self.unit_registry.degree)

        try:
            valid_speed = float(speed_token)
        except ValueError:
            print(
                "Line {}. Error in speed value {}. Couldn't convert to a number".format(
                    self.line_num, speed_token
                )
            )
            return False

        # Set speed as knots(quantity-with-unit) object
        self.set_speed(valid_speed * self.unit_registry.knot)

        try:
            if depth_token == "NaN":
                self.depth = 0.0
            else:
                self.depth = float(depth_token)
        except ValueError:
            print(
                "Line {}. Error in depth value {}. Couldn't convert to a number".format(
                    self.line_num, depth_token
                )
            )
            return False

        return True

    def parse_timestamp(self, date, time):
        if len(date) == 6:
            formatStr = "%y%m%d"
        else:
            formatStr = "%Y%m%d"

        if len(time) == 6:
            formatStr += "%H%M%S"
        else:
            formatStr += "%H%M%S.%f"

        return datetime.strptime(date + time, formatStr)

    def set_speed(self, speed):
        self.speed = speed

    def set_heading(self, heading: quantity):
        self.heading = heading

    def set_latitude(self):
        pass

    def set_longitude(self):
        pass

    def get_line_number(self):
        return self.line_num

    def get_timestamp(self):
        return self.timestamp

    def get_platform(self):
        return self.vessel

    def get_symbology(self):
        return self.symbology

    def get_latitude(self):
        return self.latitude

    def get_longitude(self):
        return self.longitude

    def get_heading(self):
        return self.heading

    def get_speed(self):
        return self.speed

    def get_depth(self):
        return self.depth

    def get_text_label(self):
        return self.text_label



import os
import unittest

from pepys_import.core.formats.repl_file import REPFile

FILE_PATH = os.path.dirname(__file__)
TEST_DATA_PATH = os.path.join(FILE_PATH, "sample_data", "rep_files")
TEST_FILE = os.path.join(TEST_DATA_PATH, "rep_test1.rep")
BROKEN_FILE = os.path.join(TEST_DATA_PATH, "rep_test2.rep")


class BasicTests(unittest.TestCase):
    def test_file_not_found(self):
        exception = False
        try:
            REPFile("non_existing.rep")
        except IOError:
            exception = True

        self.assertTrue(exception)

    def test_get_file_type(self):
        rep_file = REPFile(TEST_FILE)
        self.assertEqual("REP", rep_file.get_data_file_type())

    def test_get_file_name(self):
        rep_file = REPFile(TEST_FILE)
        self.assertEqual(TEST_FILE, rep_file.get_data_file_name())

    def test_get_all_lines(self):
        rep_file = REPFile(TEST_FILE)
        self.assertEqual(8, len(rep_file.get_lines()))

    def test_file_types(self):
        rep_file = REPFile(TEST_FILE)
        self.assertEqual("REP", rep_file.get_data_file_type())
        self.assertEqual(TEST_FILE, rep_file.get_data_file_name())

    def test_file_parse_error(self):
        exception = False
        try:
            REPFile(BROKEN_FILE)
        except Exception:
            exception = True

        self.assertTrue(exception)


if __name__ == "__main__":
    unittest.main()



from sqlalchemy import Column, Integer, String, Boolean
from sqlalchemy.dialects.sqlite import DATETIME
from sqlalchemy.dialects.sqlite import REAL
import uuid

from .db_base import base_sqlite as base
from .db_status import TableTypes
from .uuid import UUID


def map_uuid_type(val):
    # sql does not need to map to string
    return val


class Entry(base):
    __tablename__ = "Entry"
    table_type = TableTypes.METADATA

    entry_id = Column(Integer, primary_key=True)
    table_type_id = Column(Integer, nullable=False)
    created_user = Column(Integer)


class TableType(base):
    __tablename__ = "TableTypes"
    table_type = TableTypes.METADATA

    table_type_id = Column(Integer, nullable=False, primary_key=True)
    name = Column(String(150))


class SensorType(base):
    __tablename__ = "SensorTypes"
    table_type = TableTypes.REFERENCE

    sensor_type_id = Column(Integer, primary_key=True)
    # TODO: does this, or other string limits need checking or validating on file import?
    name = Column(String(150))


class Sensor(base):
    __tablename__ = "Sensors"
    table_type = TableTypes.METADATA

    # These only needed for tables referenced by Entry table
    table_type_id = 2
    tableName = "Sensor"

    sensor_id = Column(Integer, primary_key=True)
    name = Column(String(150), nullable=False)
    sensor_type_id = Column(Integer, nullable=False)
    platform_id = Column(Integer, nullable=False)


class PlatformType(base):
    __tablename__ = "PlatformTypes"
    table_type = TableTypes.REFERENCE

    platform_type_id = Column(Integer, primary_key=True)
    # TODO: does this, or other string limits need checking or validating on file import?
    name = Column(String(150))
    # TODO: add relationships and ForeignKey entries to auto-create Entry ids


class Platform(base):
    __tablename__ = "Platforms"
    table_type = TableTypes.METADATA

    # These only needed for tables referenced by Entry table
    table_type_id = 1
    tableName = "Platforms"

    platform_id = Column(Integer, primary_key=True)
    # TODO: does this, or other string limits need checking or validating on file import?
    name = Column(String(150))
    platform_type_id = Column(Integer, nullable=False)
    host_platform_id = Column(Integer)
    nationality_id = Column(Integer, nullable=False)
    # TODO: add relationships and ForeignKey entries to auto-create Entry ids

    privacy_id = Column(Integer, nullable=False)


class DatafileType(base):
    __tablename__ = "DatafileTypes"
    table_type = TableTypes.REFERENCE

    datafile_type_id = Column(Integer, primary_key=True)
    # TODO: does this, or other string limits need checking or validating on file import?
    name = Column(String(150), nullable=False)


class Datafile(base):
    __tablename__ = "Datafiles"
    table_type = TableTypes.METADATA

    # These only needed for tables referenced by Entry table
    table_type_id = 4
    tableName = "Datafiles"

    datafile_id = Column(Integer, primary_key=True)
    # TODO: does this, or other string limits need checking or validating on file import?
    simulated = Column(Boolean)
    reference = Column(String(150))
    url = Column(String(150))
    privacy_id = Column(Integer, nullable=False)
    datafile_type_id = Column(Integer, nullable=False)
    # TODO: add relationships and ForeignKey entries to auto-create Entry ids


class State(base):
    __tablename__ = "States"
    table_type = TableTypes.MEASUREMENT

    # These only needed for tables referenced by Entry table
    table_type_id = 3
    tableName = "States"

    state_id = Column(Integer, primary_key=True)
    time = Column(DATETIME, nullable=False)
    sensor_id = Column(Integer, nullable=False)
    # location = Column(Geometry(geometry_type='POINT', srid=4326))
    location = Column(String(150), nullable=False)
    heading = Column(REAL)
    course = Column(REAL)
    speed = Column(REAL)
    datafile_id = Column(Integer, nullable=False)
    privacy_id = Column(Integer)


class Nationality(base):
    __tablename__ = "Nationalities"
    table_type = TableTypes.REFERENCE

    nationality_id = Column(Integer, primary_key=True)
    name = Column(String(150), nullable=False)


class Privacy(base):
    __tablename__ = "Privacies"
    table_type = TableTypes.REFERENCE

    privacy_id = Column(Integer, primary_key=True)
    name = Column(String(150), nullable=False)



from enum import Enum


class TableTypes(Enum):
    METADATA = 1
    MEASUREMENT = 2
    REFERENCE = 3


class DBStatus:
    def __init__(self, data_store, table_types):
        self.data_store = data_store
        self.table_types = table_types
        self.status = None

    # get current table stats, store and return status
    def get_status(self):
        self.status = self.data_store.get_table_type_data(self.table_types)
        return self.status

    # print current stats, plus diff to supplied stats if passed
    def print_status(self, prev_status=None):
        max_length = len(max(self.status, key=len)) + 1
        print("{:<{}} {:<4} {:<4}".format("Table", max_length, "Num", "Diff"))
        for table in self.status:
            print(
                f"{table:<{max_length}} {self.status[table]:<4} "
                f"{self.calculate_diff(table, prev_status):<4}"
            )

    def calculate_diff(self, table, prev_status):
        if not prev_status or table not in prev_status or table not in self.status:
            return "-"
        return self.status[table] - prev_status[table]



from .state import State


# TODO: have base class to ensure filename and type are setup and returned
class REPFile:
    def __init__(self, filepath):
        self.filepath = filepath
        self.lines = []

        # TODO: it is not Pythonic way to use counter
        with open(filepath, "r") as file:
            line_number = 0
            for line in file:
                line_number += 1

                if len(line) == 0 or line[0] == ";":
                    continue

                repL = State(line_number, line)
                if not repL.parse():
                    raise Exception(
                        "failed parsing REP file {} line {}".format(
                            filepath, line_number
                        )
                    )

                self.lines.append(repL)

    def get_lines(self):
        return self.lines

    # TODO: could organise these values differently, eg. use registry of
    #  importers which also defines file type
    # although this works well for encapsulation
    @staticmethod
    def get_data_file_type():
        return "REP"

    def get_data_file_name(self):
        # TODO: should this be just the filename? or the full absolute or
        #  relative path supplied?
        return self.filepath



from sqlalchemy import types
from sqlalchemy.types import Binary
import uuid


class UUID(types.TypeDecorator):
    impl = Binary

    def __init__(self):
        self.impl.length = 16
        types.TypeDecorator.__init__(self, length=self.impl.length)

    @staticmethod
    def process_bind_param(value, dialect=None):
        if value and isinstance(value, uuid.UUID):
            return value.bytes
        elif value and not isinstance(value, uuid.UUID):
            raise ValueError(f"value {value} is not a valid uuid.UUID")
        else:
            return None

    @staticmethod
    def process_result_value(value, dialect=None):
        if value:
            return uuid.UUID(bytes=value)
        else:
            return None

    @staticmethod
    def is_mutable():
        return False


#
#
# id_column_name = "id"
#
# def id_column():
#     import uuid
#     return Column(id_column_name,UUID(),primary_key=True,default=uuid.uuid4)



class Location:
    def __init__(self, degrees, minutes, seconds, hemisphere):
        self.degrees = degrees
        self.minutes = minutes
        self.seconds = seconds
        self.hemisphere = hemisphere

    def __repr__(self):
        return (
            "("
            + str(self.degrees)
            + ", "
            + str(self.minutes)
            + ", "
            + str(self.seconds)
            + ", "
            + self.hemisphere
            + ")"
        )

    def parse(self):
        try:
            self.degrees = float(self.degrees)
        except ValueError:
            print(
                f"Error in degrees value {self.degrees}. Couldn't convert to a number"
            )
            return False

        try:
            self.minutes = float(self.minutes)
        except ValueError:
            print(
                f"Error in minutes value {self.minutes}. Couldn't convert to a number"
            )
            return False

        try:
            self.seconds = float(self.seconds)
        except ValueError:
            print(
                f"Error in seconds value {self.seconds}. Couldn't convert to a number"
            )
            return False

        if self.hemisphere not in ("N", "S", "E", "W"):
            print(
                f"Error in hemisphere value {self.hemisphere}. "
                f"Should be one of N, S, E or W"
            )
            return False

        return True



import os
import csv


def import_from_csv(data_store, path, files):
    for file in sorted(files):
        # split file into filename and extension
        table_name, _ = os.path.splitext(file)
        possible_method = "add_to_" + table_name.lower().replace(" ", "_")
        method_to_call = getattr(data_store, possible_method, None)
        if method_to_call:
            with open(os.path.join(path, file), "r") as f:
                reader = csv.reader(f)
                # skip header
                _ = next(reader)
                with data_store.session_scope() as session:
                    for row in reader:
                        method_to_call(*row)
        else:
            print(f"Method({possible_method}) not found!")



from .data_resolver import DataResolver


class DefaultResolver(DataResolver):
    # Hardcoded default values
    default_platform_type = "Warship"
    default_nationality = "UK"
    default_sensor_type = "Position"
    default_privacy = "Private"

    def resolve_platform(
        self, data_store, platformName, platform_type_str, nationality_str, privacy_str
    ):
        # needs to establish defaults for platform_type, nationality
        if not platform_type_str:
            platform_type_str = self.default_platform_type
        platform_type = data_store.search_platform_type(platform_type_str)
        if not platform_type:
            platform_type = data_store.add_to_platform_types(platform_type_str)

        if not nationality_str:
            nationality_str = self.default_nationality
        nationality = data_store.search_nationality(nationality_str)
        if not nationality:
            nationality = data_store.add_to_nationalities(nationality_str)

        if not privacy_str:
            privacy_str = self.default_privacy
        privacy = data_store.search_nationality(privacy_str)
        if not privacy:
            privacy = data_store.add_to_privacies(privacy_str)

        return platformName, platform_type, nationality, privacy

    def resolve_sensor(self, data_store, sensorName):
        # needs to establish defaults for sensorType

        sensor_type = data_store.search_sensor_type(self.default_sensor_type)
        if not sensor_type:
            sensor_type = data_store.add_to_sensor_types(self.default_sensor_type)

        return sensorName, sensor_type

    def resolve_privacy(self, data_store, table_type_id, table_name):
        # needs to establish defaults for privacy

        privacy = data_store.search_privacy(self.default_privacy)
        if not privacy:
            privacy = data_store.add_to_privacies(self.default_privacy)

        return table_type_id, privacy



from abc import ABC, abstractmethod


class DataResolver(ABC):
    @abstractmethod
    def resolve_platform(
        self, data_store, platform_name, platform_type_str, nationality_str, privacy_str
    ):
        # Implementation method should return any data necessary to create a
        # platform
        # Currently: platformName, platformType, nationality. Probably
        # hostPlatform when that is needed
        pass

    @abstractmethod
    def resolve_sensor(self, data_store, sensor_name):
        # Implementation method should return any data necessary to create a
        # sensor
        # Currently: sensorName, sensorType
        pass

    @abstractmethod
    def resolve_privacy(self, data_store, table_type_id, table_name):
        # Implementation method should return any data necessary to create a
        # privacy
        # Currently: tabletypeId, privacyName
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = []

setup_requirements = []

test_requirements = []

setup(
    author="Ian Mayo",
    author_email="ian@planetmayo.com",
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Python library to handle database ingest of geospatial track (and other) data ",
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="pepys_import",
    name="pepys_import",
    packages=find_packages(include=["pepys_import", "pepys_import.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/pepys_import",
    version='0.0.2',
    zip_safe=False,
)



import unittest
import os
from datetime import datetime

from pepys_import.core.store.data_store import DataStore
from unittest import TestCase

FILE_PATH = os.path.dirname(__file__)
TEST_DATA_PATH = os.path.join(FILE_PATH, "sample_data", "csv_files")


class TestDataStorePopulate(TestCase):
    def setUp(self):
        self.store = DataStore("", "", "", 0, ":memory:", db_type="sqlite")
        self.store.initialise()

    def tearDown(self):
        pass

    def test_populate_reference(self):
        """Test whether CSVs successfully imported to SQLite"""

        # Check tables are created but empty
        with self.store.session_scope() as session:
            nationalities = self.store.get_nationalities()
            platform_types = self.store.get_platform_types()

        # There must be no entities at the beginning
        self.assertEqual(len(nationalities), 0)
        self.assertEqual(len(platform_types), 0)

        # Import CSVs to the related tables
        with self.store.session_scope() as session:
            self.store.populate_reference(TEST_DATA_PATH)

        # Check tables filled with correct data
        with self.store.session_scope() as session:
            nationalities = self.store.get_nationalities()
            platform_types = self.store.get_platform_types()
            nationality_object = self.store.search_nationality("UNITED KINGDOM")
            platform_type_object = self.store.search_platform_type("TYPE-1")

            # Check whether they are not empty anymore and filled with correct data
            self.assertNotEqual(len(nationalities), 0)
            self.assertNotEqual(len(platform_types), 0)

            self.assertIn(nationality_object.name, "UNITED KINGDOM")
            self.assertIn(platform_type_object.name, "TTYPE-1")

    def test_populate_metadata(self):
        # reference tables must be filled first
        with self.store.session_scope() as session:
            self.store.populate_reference(TEST_DATA_PATH)

        # get all table values
        with self.store.session_scope() as session:
            platforms = self.store.get_platforms()
            datafiles = self.store.get_datafiles()
            sensors = self.store.get_sensors()

        # There must be no entities at the beginning
        self.assertEqual(len(platforms), 0)
        self.assertEqual(len(datafiles), 0)
        self.assertEqual(len(sensors), 0)

        # Import CSVs to the related tables
        with self.store.session_scope() as session:
            self.store.populate_metadata(TEST_DATA_PATH)

        with self.store.session_scope() as session:
            platforms = self.store.get_platforms()
            datafiles = self.store.get_datafiles()
            sensors = self.store.get_sensors()

            platform_object = self.store.search_platform("PLATFORM-1")
            datafile_object = self.store.search_datafile("DATAFILE-1")
            sensor_object = self.store.search_sensor("SENSOR-1")

            # Check whether they are not empty anymore and filled with correct data
            self.assertNotEqual(len(platforms), 0)
            self.assertNotEqual(len(datafiles), 0)
            self.assertNotEqual(len(sensors), 0)

            # The following assertions filter objects by foreign key ids and
            # compares values with the data from CSV

            # Platform Object: PLATFORM-1, UNITED KINGDOM, TYPE-1, PRIVACY-1
            nationality = (
                self.store.session.query(self.store.db_classes.Nationality)
                .filter_by(nationality_id=platform_object.nationality_id)
                .first()
            )
            self.assertEqual(nationality.name, "UNITED KINGDOM")
            platform_type = (
                self.store.session.query(self.store.db_classes.PlatformType)
                .filter_by(platform_type_id=platform_object.platform_type_id)
                .first()
            )
            self.assertEqual(platform_type.name, "TYPE-1")
            privacy = (
                self.store.session.query(self.store.db_classes.Privacy)
                .filter_by(privacy_id=platform_object.privacy_id)
                .first()
            )
            self.assertEqual(privacy.name, "PRIVACY-1")

            # Datafile Object: DATAFILE-1, True, PRIVACY-1, DATAFILE-TYPE-1
            self.assertEqual(datafile_object.simulated, True)
            privacy = (
                self.store.session.query(self.store.db_classes.Privacy)
                .filter_by(privacy_id=datafile_object.privacy_id)
                .first()
            )
            self.assertEqual(privacy.name, "PRIVACY-1")
            datafile_type = (
                self.store.session.query(self.store.db_classes.DatafileType)
                .filter_by(datafile_type_id=datafile_object.datafile_type_id)
                .first()
            )
            self.assertEqual(datafile_type.name, "DATAFILE-TYPE-1")

            # Sensor Object: SENSOR-1, SENSOR-TYPE-1, PLATFORM-1
            sensor_type = (
                self.store.session.query(self.store.db_classes.SensorType)
                .filter_by(sensor_type_id=sensor_object.sensor_type_id)
                .first()
            )
            self.assertEqual(sensor_type.name, "SENSOR-TYPE-1")
            platform = (
                self.store.session.query(self.store.db_classes.Platform)
                .filter_by(platform_id=sensor_object.platform_id)
                .first()
            )
            self.assertEqual(platform.name, "PLATFORM-1")

    def test_populate_measurement(self):
        # reference and metadata tables must be filled first
        with self.store.session_scope() as session:
            self.store.populate_reference(TEST_DATA_PATH)
            self.store.populate_metadata(TEST_DATA_PATH)

        # get all table values
        with self.store.session_scope() as session:
            states = self.store.get_states()

        # There must be no entities at the beginning
        self.assertEqual(len(states), 0)

        # Import CSVs to the related tables
        with self.store.session_scope() as session:
            self.store.populate_measurement(TEST_DATA_PATH)

        # Check tables filled with correct data
        with self.store.session_scope() as session:
            states = self.store.get_states()
            first_state = self.store.session.query(self.store.db_classes.State).first()

            # Check whether they are not empty anymore and filled with correct data
            self.assertNotEqual(len(states), 0)

            # The following assertions filter objects by foreign key ids and
            # compares values with the data from CSV

            # first_state = 2019-01-12 12:10:00, SENSOR-1, DATAFILE-1,46.000 32.000,,,,
            # PRIVACY-1
            self.assertEqual(
                first_state.time,
                datetime.strptime("2019-01-12 12:10:00", "%Y-%m-%d %H:%M:%S"),
            )
            self.assertEqual(first_state.location, "46.000 32.000")
            privacy = (
                self.store.session.query(self.store.db_classes.Privacy)
                .filter_by(privacy_id=first_state.privacy_id)
                .first()
            )
            self.assertEqual(privacy.name, "PRIVACY-1")
            datafile = (
                self.store.session.query(self.store.db_classes.Datafile)
                .filter_by(datafile_id=first_state.datafile_id)
                .first()
            )
            self.assertEqual(datafile.reference, "DATAFILE-1")
            sensor = (
                self.store.session.query(self.store.db_classes.Sensor)
                .filter_by(sensor_id=first_state.sensor_id)
                .first()
            )
            self.assertEqual(sensor.name, "SENSOR-1")


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from unittest import TestCase
from sqlalchemy import inspect, MetaData, Table


from pepys_import.core.debug.support_methods import SupportMethods
from pepys_import.core.store.data_store import DataStore
from pepys_import.core.formats.repl_file import REPFile
from pepys_import.core.store.sqlite_db import State


FILE_PATH = os.path.dirname(__file__)
TEST_DATA_PATH = os.path.join(FILE_PATH, "sample_data", "rep_files")
TEST_FILE = os.path.join(TEST_DATA_PATH, "rep_test1.rep")
BROKEN_FILE = os.path.join(TEST_DATA_PATH, "rep_test2.rep")


class TestLoadReplay(TestCase):
    def test_load_replay(self):
        """Test  whether we can load REP data"""
        data_store = DataStore("", "", "", 0, ":memory:", db_type="sqlite")

        # creating database from schema
        data_store.initialise()

        rep_file = REPFile(TEST_FILE)
        self.assertEqual("REP", rep_file.get_data_file_type())

        with data_store.session_scope() as session:
            datafile = session.add_to_datafile_from_rep(
                rep_file.get_data_file_name(), rep_file.get_data_file_type()
            )
            for repLine in rep_file.get_lines():
                platform = session.add_to_platforms_from_rep(
                    repLine.get_platform(), "Fisher", "UK", "Public"
                )

                sensor = session.add_to_sensors_from_rep(
                    platform.name + "_GPS", platform
                )
                session.add_to_states_from_rep(
                    repLine.get_timestamp(),
                    datafile,
                    sensor,
                    repLine.get_latitude(),
                    repLine.get_longitude(),
                    repLine.get_heading(),
                    repLine.get_speed(),
                )

        support = SupportMethods()
        print("Found:" + str(support.count_states(data_store)))

        support.list_all(data_store)


if __name__ == "__main__":
    unittest.main()
